"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createApp = exports.DownloadError = void 0;
const async_retry_1 = __importDefault(require("async-retry"));
const chalk_1 = __importDefault(require("chalk"));
const cpy_1 = __importDefault(require("cpy"));
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const examples_1 = require("./helpers/examples");
const make_dir_1 = require("./helpers/make-dir");
const git_1 = require("./helpers/git");
const install_1 = require("./helpers/install");
const is_folder_empty_1 = require("./helpers/is-folder-empty");
const is_online_1 = require("./helpers/is-online");
const is_writeable_1 = require("./helpers/is-writeable");
class DownloadError extends Error {
}
exports.DownloadError = DownloadError;
async function createApp({ appPath, packageManager, example, examplePath, typescript, }) {
    let repoInfo;
    const template = 'default';
    if (example) {
        let repoUrl;
        try {
            repoUrl = new URL(example);
        }
        catch (error) {
            if (error.code !== 'ERR_INVALID_URL') {
                console.error(error);
                process.exit(1);
            }
        }
        if (repoUrl) {
            if (repoUrl.origin !== 'https://github.com') {
                console.error(`Invalid URL: ${chalk_1.default.red(`"${example}"`)}. Only GitHub repositories are supported. Please use a GitHub URL and try again.`);
                process.exit(1);
            }
            repoInfo = await (0, examples_1.getRepoInfo)(repoUrl, examplePath);
            if (!repoInfo) {
                console.error(`Found invalid GitHub URL: ${chalk_1.default.red(`"${example}"`)}. Please fix the URL and try again.`);
                process.exit(1);
            }
            const found = await (0, examples_1.hasRepo)(repoInfo);
            if (!found) {
                console.error(`Could not locate the repository for ${chalk_1.default.red(`"${example}"`)}. Please check that the repository exists and try again.`);
                process.exit(1);
            }
        }
        else if (example !== '__internal-testing-retry') {
            const found = await (0, examples_1.existsInRepo)(example);
            if (!found) {
                console.error(`Could not locate an example named ${chalk_1.default.red(`"${example}"`)}. It could be due to the following:\n`, `1. Your spelling of example ${chalk_1.default.red(`"${example}"`)} might be incorrect.\n`, `2. You might not be connected to the internet or you are behind a proxy.`);
                process.exit(1);
            }
        }
    }
    const root = path_1.default.resolve(appPath);
    if (!(await (0, is_writeable_1.isWriteable)(path_1.default.dirname(root)))) {
        console.error('The application path is not writable, please check folder permissions and try again.');
        console.error('It is likely you do not have write permissions for this folder.');
        process.exit(1);
    }
    const appName = path_1.default.basename(root);
    await (0, make_dir_1.makeDir)(root);
    if (!(0, is_folder_empty_1.isFolderEmpty)(root, appName)) {
        process.exit(1);
    }
    const useYarn = packageManager === 'yarn';
    const isOnline = !useYarn || (await (0, is_online_1.getOnline)());
    const originalDirectory = process.cwd();
    console.log(`Creating a new steedos app in ${chalk_1.default.green(root)}.`);
    console.log();
    process.chdir(root);
    const packageJsonPath = path_1.default.join(root, 'package.json');
    let hasPackageJson = false;
    if (example) {
        try {
            if (repoInfo) {
                const repoInfo2 = repoInfo;
                console.log(`Downloading files from repo ${chalk_1.default.cyan(example)}. This might take a moment.`);
                console.log();
                await (0, async_retry_1.default)(() => (0, examples_1.downloadAndExtractRepo)(root, repoInfo2), {
                    retries: 3,
                });
            }
            else {
                console.log(`Downloading files for example ${chalk_1.default.cyan(example)}. This might take a moment.`);
                console.log();
                await (0, async_retry_1.default)(() => (0, examples_1.downloadAndExtractExample)(root, example), {
                    retries: 3,
                });
            }
        }
        catch (reason) {
            function isErrorLike(err) {
                return (typeof err === 'object' &&
                    err !== null &&
                    typeof err.message === 'string');
            }
            throw new DownloadError(isErrorLike(reason) ? reason.message : reason + '');
        }
        const ignorePath = path_1.default.join(root, '.gitignore');
        if (!fs_1.default.existsSync(ignorePath)) {
            fs_1.default.copyFileSync(path_1.default.join(__dirname, '..', 'templates', template, 'gitignore'), ignorePath);
        }
        const tsconfigPath = path_1.default.join(root, 'tsconfig.json');
        if (fs_1.default.existsSync(tsconfigPath)) {
            fs_1.default.copyFileSync(path_1.default.join(__dirname, '..', 'templates', 'typescript', 'next-env.d.ts'), path_1.default.join(root, 'next-env.d.ts'));
        }
        hasPackageJson = fs_1.default.existsSync(packageJsonPath);
        if (hasPackageJson) {
            console.log('Installing packages. This might take a couple of minutes.');
            console.log();
            await (0, install_1.install)(root, null, { packageManager, isOnline });
            console.log();
        }
    }
    else {
        console.log(chalk_1.default.bold(`Using ${packageManager}.`));
        const packageJson = {
            name: appName,
            version: '0.1.0',
            private: true,
            scripts: {
                start: "moleculer-runner services/*/package.service.js --hot --repl",
                repl: "moleculer-runner --repl",
            }
        };
        fs_1.default.writeFileSync(path_1.default.join(root, 'package.json'), JSON.stringify(packageJson, null, 2) + os_1.default.EOL);
        const installFlags = { packageManager, isOnline };
        const dependencies = [
            '@steedos/service-package-loader'
        ];
        const devDependencies = [
            'moleculer-repl'
        ];
        if (typescript) {
            devDependencies.push('typescript');
        }
        if (dependencies.length) {
            console.log();
            console.log('Installing dependencies:');
            for (const dependency of dependencies) {
                console.log(`- ${chalk_1.default.cyan(dependency)}`);
            }
            console.log();
            await (0, install_1.install)(root, dependencies, installFlags);
        }
        if (devDependencies.length) {
            console.log();
            console.log('Installing devDependencies:');
            for (const devDependency of devDependencies) {
                console.log(`- ${chalk_1.default.cyan(devDependency)}`);
            }
            console.log();
            const devInstallFlags = Object.assign({ devDependencies: true }, installFlags);
            await (0, install_1.install)(root, devDependencies, devInstallFlags);
        }
        console.log();
        await (0, cpy_1.default)(['**'], root, {
            dot: true,
            parents: true,
            cwd: path_1.default.join(__dirname, '..', 'templates', template),
        });
    }
    if ((0, git_1.tryGitInit)(root)) {
        console.log('Initialized a git repository.');
        console.log();
    }
    let cdpath;
    if (path_1.default.join(originalDirectory, appName) === appPath) {
        cdpath = appName;
    }
    else {
        cdpath = appPath;
    }
    console.log(`${chalk_1.default.green('Success!')} Created ${appName} at ${appPath}`);
    console.log();
}
exports.createApp = createApp;
//# sourceMappingURL=create-app.js.map