#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const commander_1 = __importDefault(require("commander"));
const path_1 = __importDefault(require("path"));
const prompts_1 = __importDefault(require("prompts"));
const update_check_1 = __importDefault(require("update-check"));
const create_app_1 = require("./create-app");
const get_pkg_manager_1 = require("./helpers/get-pkg-manager");
const validate_pkg_1 = require("./helpers/validate-pkg");
const packageJson = require('../package.json');
let projectPath = '';
const program = new commander_1.default.Command(packageJson.name)
    .version(packageJson.version)
    .arguments('<project-directory>')
    .usage(`${chalk_1.default.green('<project-directory>')} [options]`)
    .action((name) => {
    projectPath = name;
})
    .option('--ts, --typescript', `

  Initialize as a TypeScript project.
`)
    .option('--use-npm', `

  Explicitly tell the CLI to bootstrap the app using npm
`)
    .option('--use-pnpm', `

  Explicitly tell the CLI to bootstrap the app using pnpm
`)
    .option('-e, --example [name]|[github-url]', `

  An example to bootstrap the app with. You can use an example name
  from the official Next.js repo or a GitHub URL. The URL can use
  any branch and/or subdirectory
`)
    .option('--example-path <path-to-example>', `

  In a rare case, your GitHub URL might contain a branch name with
  a slash (e.g. bug/fix-1) and the path to the example (e.g. foo/bar).
  In this case, you must specify the path to the example separately:
  --example-path foo/bar
`)
    .allowUnknownOption()
    .parse(process.argv);
async function run() {
    if (typeof projectPath === 'string') {
        projectPath = projectPath.trim();
    }
    if (!projectPath) {
        const res = await (0, prompts_1.default)({
            type: 'text',
            name: 'path',
            message: 'What is your project named?',
            initial: 'my-app',
            validate: (name) => {
                const validation = (0, validate_pkg_1.validateNpmName)(path_1.default.basename(path_1.default.resolve(name)));
                if (validation.valid) {
                    return true;
                }
                return 'Invalid project name: ' + validation.problems[0];
            },
        });
        if (typeof res.path === 'string') {
            projectPath = res.path.trim();
        }
    }
    if (!projectPath) {
        console.log('\nPlease specify the project directory:\n' +
            `  ${chalk_1.default.cyan(program.name())} ${chalk_1.default.green('<project-directory>')}\n` +
            'For example:\n' +
            `  ${chalk_1.default.cyan(program.name())} ${chalk_1.default.green('my-steedos-app')}\n\n` +
            `Run ${chalk_1.default.cyan(`${program.name()} --help`)} to see all options.`);
        process.exit(1);
    }
    const resolvedProjectPath = path_1.default.resolve(projectPath);
    const projectName = path_1.default.basename(resolvedProjectPath);
    const { valid, problems } = (0, validate_pkg_1.validateNpmName)(projectName);
    if (!valid) {
        console.error(`Could not create a project called ${chalk_1.default.red(`"${projectName}"`)} because of npm naming restrictions:`);
        problems.forEach((p) => console.error(`    ${chalk_1.default.red.bold('*')} ${p}`));
        process.exit(1);
    }
    if (program.example === true) {
        console.error('Please provide an example name or url, otherwise remove the example option.');
        process.exit(1);
    }
    const packageManager = !!program.useNpm
        ? 'npm'
        : !!program.usePnpm
            ? 'pnpm'
            : (0, get_pkg_manager_1.getPkgManager)();
    const example = typeof program.example === 'string' ? program.example.trim() : undefined;
    try {
        await (0, create_app_1.createApp)({
            appPath: resolvedProjectPath,
            packageManager,
            example: example && example !== 'default' ? example : undefined,
            examplePath: program.examplePath,
            typescript: program.typescript,
        });
    }
    catch (reason) {
        if (!(reason instanceof create_app_1.DownloadError)) {
            throw reason;
        }
        const res = await (0, prompts_1.default)({
            type: 'confirm',
            name: 'builtin',
            message: `Could not download "${example}" because of a connectivity issue between your machine and GitHub.\n` +
                `Do you want to use the default template instead?`,
            initial: true,
        });
        if (!res.builtin) {
            throw reason;
        }
        await (0, create_app_1.createApp)({
            appPath: resolvedProjectPath,
            packageManager,
            typescript: program.typescript,
        });
    }
}
const update = (0, update_check_1.default)(packageJson).catch(() => null);
async function notifyUpdate() {
    try {
        const res = await update;
        if (res === null || res === void 0 ? void 0 : res.latest) {
            const pkgManager = (0, get_pkg_manager_1.getPkgManager)();
            console.log(chalk_1.default.yellow.bold('A new version of `create-steedos-app` is available!') +
                '\n' +
                'You can update by running: ' +
                chalk_1.default.cyan(pkgManager === 'yarn'
                    ? 'yarn global add create-steedos-app'
                    : `${pkgManager} install --global create-steedos-app`) +
                '\n');
        }
        process.exit();
    }
    catch (_a) {
    }
}
run()
    .then(notifyUpdate)
    .catch(async (reason) => {
    console.log();
    console.log('Aborting installation.');
    if (reason.command) {
        console.log(`  ${chalk_1.default.cyan(reason.command)} has failed.`);
    }
    else {
        console.log(chalk_1.default.red('Unexpected error. Please report it as a bug:') + '\n', reason);
    }
    console.log();
    await notifyUpdate();
    process.exit(1);
});
//# sourceMappingURL=index.js.map